from typing import Dict

import numpy as np
import pandas as pd

from mu_map.data.prepare import headers
from mu_map.data.split import split_csv


def body_mass_index(weight: np.ndarray, size: np.ndarray) -> np.ndarray:
    return weight / (size**2)


def evaluate_protocols(data: pd.DataFrame) -> Dict[str, int]:
    data[headers.datetime_acquisition] = pd.to_datetime(
        data[headers.datetime_acquisition]
    )

    protocols = {"stress": 0, "rest": 0, "stress/rest": 0, "rest/stress": 0}
    for patient_id in data[headers.patient_id].unique():
        _data = data[data[headers.patient_id] == patient_id]
        _data = _data.sort_values(by=headers.datetime_acquisition)
        protocol = _data[headers.protocol].unique()
        protocol = list(map(lambda _str: _str.lower(), protocol))
        protocol = "/".join(protocol)
        protocols[protocol] += 1
    return protocols


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(
        description="Evaluate dataset patient stats.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument("--meta_csv", type=str, default="data/second/meta.csv")
    parser.add_argument("--split_csv", type=str, default="data/second/split.csv")
    args = parser.parse_args()
    print(args)
    print()

    data = pd.read_csv(args.meta_csv)
    _from = data[headers.datetime_acquisition].min().split(" ")[0]
    _to = data[headers.datetime_acquisition].max().split(" ")[0]
    print(f"Scans were performed from {_from} to {_to}")
    protocols = evaluate_protocols(data)
    print("Protocols:")
    for protocol, count in protocols.items():
        print(f" - {protocol:>12}: {str(count):>3}")
    print()

    splits = split_csv(data, args.split_csv)
    for split_name, split in splits.items():
        print(f"{split_name[0].upper()}{split_name[1:]}:")

        n_studies = len(split)
        n_stress = len(split[split[headers.protocol] == "Stress"])
        n_rest = len(split[split[headers.protocol] == "Rest"])
        print(f" -    Studies: {n_studies}, Stress/Rest: {n_stress}/{n_rest}")
        n_patients = len(split[headers.patient_id].unique())
        _group = split.groupby(headers.patient_id).count()[headers.id]
        n_studies_one = (_group == 1).sum()
        n_studies_two = (_group == 2).sum()
        n_studies_three = (_group == 3).sum()
        _split = split.drop_duplicates(headers.patient_id)
        n_males = len(_split[_split[headers.sex] == "M"])
        n_females = len(_split[_split[headers.sex] == "F"])
        print(
            f" -   Patients: {n_patients}, M/F: {n_males}/{n_females}, Number of studies (1, 2, 3): [{n_studies_one}, {n_studies_two}, {n_studies_three}]"
        )

        age = split[headers.age]
        height = split[headers.size]
        weight = split[headers.weight]
        bmi = body_mass_index(weight, height)

        age = age.astype(int)
        weight = weight.astype(int)
        height = np.round(height * 100).astype(int)
        bmi = np.round(bmi, 2)

        for stat, label in [
            (age, "Age"),
            (height, "Height"),
            (weight, "Weight"),
            (bmi, "BMI"),
        ]:
            _min = f"{stat.min():.1f}"
            _max = f"{stat.max():.1f}"
            _mean = f"{stat.mean():.1f}"
            _std = f"{stat.std():.1f}"
            print(f" - {label:>10}: [{_min:>5}, {_max:>5}] - {_mean:>5}±{_std:>5}")
